# WorkflowGen hooks Mocha tests

## Requirements
- WorkflowGen configuration parameters
  - In General / Security , `Workflow engine service default identity` must contain a valid WorkflowGen user account. This will be the default user identity executing the hooks operations. This setting refers to the key `EngineServiceImpersonificationUsername` in the web.config file. e.g. `<add key="EngineServiceImpersonificationUsername" value="wfgen_admin" />`

  - In Integration / Security, `System operations allowed users (usernames separated by commas)` must also contain the same account as defined above in `Workflow engine service default identity`. This setting refers to the key `ProcessesRuntimeWebServiceAllowedUsers` in the web.config file. e.g. `<add key="ProcessesRuntimeWebServiceAllowedUsers" value="wfgen_admin" />`

  - The anonymous authentication must be enabled for the `/wfgen/hooks` web application.

## Installation

1. In the SQL Server Configuration Manager, enable the TCP/IP protocol for the MSSQLSERVER service and restart the MSSQLSERVER service.
2. Set the following environmental variables (using the command prompt). If they are not set, they will take their default value that have been assigned in the `config.test.js` file.
    - `TestDbServer`: Database connection server. 
    - `TestDbAdminUserName`: Database connection administrator username. 
    - `TestDbAdminUserPassword`: Database connection administrator password.
    - `TestDbUserName`: Database connection username. 
    - `TestDbUserPassword`: Database connection password.
    - `TestDbName`: Database name.
    - `TestDbPort`: Database connection port.
    - `TestInstallScriptPath`: The local path where the SQL creation scripts are located. The folder should contains the following script files
      - Create_WFG-V7-0_SQL_Tables.sql
      - Create_WFG-V7-0_SQL_PKeys.sql
      - Create_WFG-V7-0_SQL_FKeys.sql
      - Create_WFG-V7-0_SQL_Indexes.sql
      - Create_WFG-V7-0_SQL_Triggers.sql
      - Create_WFG-V7-0_SQL_Const.sql

3. Change the values of the `Initial Catalog`, `User ID`, and `Password` properties of the MainDbSource connection string in the `/wfgen/web.config` file to the respective `test.databaseConnection.database`, `test.databaseConnection.user`, and `test.databaseConnection.password` values specified in the `config.test.js` file. Note that the WorkflowGen site won't be functional until you reset the connection string with the previously existing values.

4. As of version 7.8.0, two complete scenarios of the `SIMPLE APPROVAL` process can be executed; the scenarios include request creation and closing, action completion, request and action dataset verification, and checking notifications. They require that email notifications be enabled and correctly set (the `EngineNotificationEnabled` and `EngineNotificationDefaultSender` parameters in the `\wfgen\web.config` file), with the SMTP delivery mode set to `Pickup directory`. The `applicationSmtpPickupDirectory` value in the `config.test.js` file must match the `ApplicationSmtpPickupDirectory` value in the `\wfgen\web.config` file. The path provided in the `applicationSmtpPickupDirectory` value must exist.

**Important notes** 
- If you are upgrading WorkflowGen with an existing `hooks` application and you've modified the default values of the parameters in the `config.test.js` file, be careful to not overwrite this configuration file with the one from the WorkflowGen upgrade pack. You should compare the new `config.test.js` from the upgrade pack with your current file and apply any changes, such as new parameters.
- Since some new dependencies libraries have been added, it's necessary to execute an `npm install` to make the built-in process tests work.

## Database scripts
Using the command prompt, go to the `hooks` application.

To create a new database for the tests, execute the following commands from the command prompt (using `npm run <command>`):
- `test:db:create`: Create the database, login, user, schema, tables, constraints, indexes and triggers.
- `test:db:init`: Delete the database records if they exist, and insert the constants and the test data sets.

After having run the tests, if you want to run them again, execute the `test:db:init` command to reinitialize the test data sets.

The following commands are also available
- `test:db:reset`: Drop and create the database and all its objects.
- `test:db:drop`: Drop the  database and all its content.

## Launch the tests
Once the database is prepared, you can run the Mocha test suites in one of the following modes:
- `Standalone` (recommended): Launches a separate web server (server.js) to host the hooks web application for the Mocha test suites.
- `WorkflowGen`: Uses the localhost WorkflowGen `hooks` web application for the Mocha test suites. If you use this mode, you should update the `test.server.port` setting in the `config.test.js` file by adding the WorkflowGen application port. The `hooks.operations` values in the `config.production.js` file must match the `hooks.operations` in the `config.test.js` file. Be aware that you should restore the previously existing operation values after the tests.

To launch the tests, execute one of the following commands in the command prompt:
- Full test: `npm test`
- Test the server module: `npm run test:server`
- Test the queries: `npm run test:queries`
- Test the mutations: `npm run test:mutations`
- Test the built-in processes: `npm run test:processes`.