var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('CancelAction', function () {

  it('should cancel an action', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelAction',
        'args': {
          'input': {
            'id': 'QWN0aW9uOjktLS0y'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });
  
  it("should return the action(ID: 'QWN0aW9uOjktLS0y') cancelled", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjktLS0y'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjktLS0y', status: 'CLOSED', subStatus: 'ABORTED' });

        done();
      });
  });

  it('should cancel an action in delegation mode', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelAction',
        'args': {
          'input': {
            'id': 'QWN0aW9uOjI3LS0tMQ==',
            'onBehalfOf': 'VXNlcjoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).not.to.be.a('null');
        res.body.onBehalfOf.should.be.equal('VXNlcjoy');
        res.body.onBehalfOfUserName.should.be.equal('mark_twain');

        done();
      });
  });

  it("should return the action(ID: 'QWN0aW9uOjI3LS0tMQ==') cancelled by a user (ID: 'VXNlcjoy') in delegation mode", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjI3LS0tMQ==',
          'onBehalfOf': 'VXNlcjoy'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjI3LS0tMQ==', status: 'CLOSED', subStatus: 'ABORTED', assigneeId: 'VXNlcjoy' });

        done();
      });
  });

  it('should cancel an action containing the clientMutationId property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelAction',
        'args': {
          'input': {
            'id': 'QWN0aW9uOjEwLS0tMg==',
            'clientMutationId': 'VXNlcjow'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        res.body.clientMutationId.should.be.equal('VXNlcjow');
        res.body.actionId.should.be.a('string');
        expect(res.body.actionId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the action(ID: 'QWN0aW9uOjEwLS0tMg==') cancelled", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjEwLS0tMg=='
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjEwLS0tMg==', status: 'CLOSED', subStatus: 'ABORTED' });

        done();
      });
  });
});
