var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('CancelRequestActionsByName', function () {
  
  it('should cancel a request actions using the name property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequestActionsByName',
        'args': {
          'input': {
            'requestNumber': 16,
            'activityName': 'Initiates'
          }
        }
      })
      .end(function (err, res) {
        console.log(err);
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionIds');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionIds.should.be.a('array');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the request actions having the activity name 'Initiates' cancelled", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjE2LS0tMQ=='
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjE2LS0tMQ==', activityId: 'QWN0aXZpdHk6My0tLTE=', status: 'CLOSED', subStatus: 'ABORTED' });

        done();
      });
  });

  it('should cancel a request actions in delegation mode using the name property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequestActionsByName',
        'args': {
          'input': {
            'requestNumber': 30,
            'activityName': 'Initiates',
            'onBehalfOf': 'VXNlcjoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('actionIds');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.actionIds.should.be.a('array');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.onBehalfOf).not.to.be.a('null');
        expect(res.body.onBehalfOfUserName).not.to.be.a('null');
        res.body.onBehalfOf.should.be.equal('VXNlcjoy');
        res.body.onBehalfOfUserName.should.be.equal('mark_twain');

        done();
      });
  });

  it("should return the request actions having the activity name 'Initiates' cancelled in delegation mode", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getAction',
        'args': {
          'id': 'QWN0aW9uOjMwLS0tMQ==',
          'onBehalfOf': 'VXNlcjoy'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateActionProperties(res.body);

        expect(res.body).to.include({ id: 'QWN0aW9uOjMwLS0tMQ==', activityId: 'QWN0aXZpdHk6My0tLTE=', status: 'CLOSED', subStatus: 'ABORTED' });

        done();
      });
  });
});
