var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CreateDelegation', function () {
  it('should create a delegation for a process participant', function (done) {
    chai.request(url_wfgen_admin)
    .post('/')
    .send({
      'operation': 'createDelegation',
      'args': {
        'input': {
          'delegatorId': 'VXNlcjox',
          'delegateeId': 'VXNlcjoy',
          'processId': 'UHJvY2Vzczoy',
          'participantId': 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NA==',
          'notifyDelegatee': false,
          'start': null,
          'end': null
        }
      }
    })
    .end(function (err, res) {
      res.should.have.status(200);
      res.should.be.json;
      res.body.should.have.property('clientMutationId');
      res.body.should.have.property('delegationId');

      expect(res.body).to.include({ delegationId:'RGVsZWdhdGlvbjo2', clientMutationId:null });
      done();
    });
  });

  it('should verify the created delegation\'s data', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
          'operation': 'getDelegation',
          'args': {
              'id': 'RGVsZWdhdGlvbjo2'
          }
      })
      .end(function (err, res) {

        res.should.have.status(200);
        res.should.be.json;

        expect(res.body).to.include({ "id": "RGVsZWdhdGlvbjo2", "delegatorId": "VXNlcjox", "delegateeId": "VXNlcjoy", "start": null, "end": null, "processId": "UHJvY2Vzczoy", "participantId": "TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NA==" });

      done();
      });
  });

  it('should create a delegation for a process participant with dates', function (done) {
    chai.request(url_wfgen_admin)
    .post('/')
    .send({
      'operation': 'createDelegation',
      'args': {
        'input': {
          'delegatorId': 'VXNlcjox',
          'delegateeId': 'VXNlcjoy',
          'processId': 'UHJvY2Vzczoz',
          'participantId': 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6OA==',
          'notifyDelegatee': false,
          'start': "2017-04-23T18:25:43.000Z",
          'end': "2017-11-23T18:25:43.000Z"
        }
      }
    })
    .end(function (err, res) {
      res.should.have.status(200);
      res.should.be.json;
      res.body.should.have.property('clientMutationId');
      res.body.should.have.property('delegationId');

      expect(res.body).to.include({ delegationId:'RGVsZWdhdGlvbjo3', clientMutationId:null });
      done();
    });
  });
  it('should verify the created delegation\'s data', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getDelegation',
        'args': {
            'id': 'RGVsZWdhdGlvbjo3'
        }
      })
      .end(function (err, res) {

        res.should.have.status(200);
        res.should.be.json;

        expect(res.body).to.include({ "id": "RGVsZWdhdGlvbjo3", "delegatorId": "VXNlcjox", "delegateeId": "VXNlcjoy", "start": "2017-04-23T18:25:43.000Z", "end": "2017-11-23T18:25:43.000Z", "processId": "UHJvY2Vzczoz", "participantId": "TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6OA==" });

      done();
      });
  });
});
