var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('CancelRequest', function () {

  it('should cancel a request using the request number', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'number': 1
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the request(number: '1') closed ", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'number': 1
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDox', number: 1, status: 'CLOSED' });

        done();
      });
  });

  it('should cancel a request using the request ID', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the request(ID: 'UmVxdWVzdDoy') closed", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDoy'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDoy', status: 'CLOSED' });

        done();
      });
  });

  it('should cancel a request with notify participant false', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDoz',
            'notifyParticipant': false
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the request(ID: 'UmVxdWVzdDoz') closed", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDoz'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDoz', status: 'CLOSED' });

        done();
      });
  });

  it('should cancel a request with notify participant true', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDo0',
            'notifyParticipant': true
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });

  it("should return the request(ID: 'UmVxdWVzdDo0') closed", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDo0'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDo0', status: 'CLOSED' });

        done();
      });
  });

  it('should cancel a request in delegation mode using onBehalfOf property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDoyOA==',
            'onBehalfOf': 'VXNlcjoy'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).not.to.be.a('null');
        res.body.onBehalfOf.should.be.equal('VXNlcjoy');
        res.body.onBehalfOfUserName.should.be.equal('mark_twain');

        done();
      });
  });

  it("should return the request(ID: 'UmVxdWVzdDoyOA==') closed by a user (ID: 'VXNlcjoy') in delegation mode", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDoyOA=='
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDoyOA==', status: 'CLOSED', abortedById: 'VXNlcjoy' });

        done();
      });
  });


  it('should cancel a request in delegation mode using onBehalfOfUserName property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDoyOQ==',
            'onBehalfOfUserName': 'mark_twain'
          }
        }

      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        res.body.onBehalfOfUserName.should.be.equal('mark_twain');

        done();
      });
  });

  it("should return the request(ID: 'UmVxdWVzdDoyOQ==') closed by a user (userName: 'mark_twain') in delegation mode", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDoyOQ=='
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDoyOQ==', status: 'CLOSED', abortedById: 'VXNlcjoy' });

        done();
      });
  });

  it('should cancel a request containing the clientMutationId property', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'cancelRequest',
        'args': {
          'input': {
            'id': 'UmVxdWVzdDo3',
            'clientMutationId': 'VXNlcjow'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('requestId');
        res.body.should.have.property('number');
        res.body.should.have.property('onBehalfOf');
        res.body.should.have.property('onBehalfOfUserName');
        res.body.clientMutationId.should.be.equal('VXNlcjow');
        res.body.requestId.should.be.a('string');
        expect(res.body.requestId).not.to.be.a('null');
        res.body.number.should.be.a('number');
        expect(res.body.onBehalfOf).to.be.a('null');
        expect(res.body.onBehalfOfUserName).to.be.a('null');

        done();
      });
  });
  
  it("should return the request(ID: 'UmVxdWVzdDo3') closed", function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getRequest',
        'args': {
          'id': 'UmVxdWVzdDo3'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateRequestProperties(res.body);

        expect(res.body).to.include({ id: 'UmVxdWVzdDo3', status: 'CLOSED' });

        done();
      });
  });
});
