var chai = require('chai');
var chaiHttp = require('chai-http');
var things = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(things);

var userId = null;

describe('RemoveGroupsFromUser', function () {

    it('should remove groups from a user', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'removeGroupsFromUser',
        'args': {
          'input': {
            'id': 'VXNlcjoz',
            'groupIds': ["R3JvdXA6Mw=="]
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    });

    it('should return the groups associated to a user', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getGroups',
            'args': {
                'userId': userId
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateGroupPageListProperties(res.body, 1);

            expect(res.body.items[0]).to.include({ id: "R3JvdXA6Mg==", name: 'Group B', description: "Group B", directoryId: "RGlyZWN0b3J5OjE=" });

            done();
        });
    });

    it('should get an error when the group does not exist in the user\'s directory', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
        'operation': 'removeGroupsFromUser',
          'args': {
            'input': {
              'id': 'VXNlcjoz',
              'groupIds': ["R3JvdXA6NA=="]
            }
          }
        })
        .end(function (err, res) {
            res.should.have.status(500);
            res.should.be.json;

            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The group ID \'4\' does not exist or is not in the same directory as the user.');

            done();
        });
    });

    it('should get an error when the array of group id is not defined', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'removeGroupsFromUser',
        'args': {
          'input': {
            'id': 'VXNlcjoz'
          }
        }
      })
      .end(function (err, res) {
          res.should.have.status(400);
          res.should.be.json;

          res.body.should.have.property('error');
          expect(res.body.error).to.contain('Group ids are required.');

          done();
      });
    });

    it('should get an error when the array of group id is empty', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'removeGroupsFromUser',
        'args': {
          'input': {
            'id': 'VXNlcjoz',
            'groupIds': []
          }
        }
      })
      .end(function (err, res) {
          res.should.have.status(400);
          res.should.be.json;

          res.body.should.have.property('error');
          expect(res.body.error).to.contain('Group ids are required.');

          done();
      });
    });

});
