var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

var userId = null;

describe('UpdateUser', function () {
  
    it('should update the user with a different username', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'updateUser',
        'args': {
          'input': {
            'id': 'VXNlcjo1',
            'userName': 'jane_doe_updated',
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    });

    it('should return the updated user with the new username ', function (done) {
      if (userId === null) {
        this.skip();
      }
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateUserProperties(res.body);

        expect(res.body).to.include({ id: userId, userName: 'jane_doe_updated', lastName: "Doe", directoryId: "RGlyZWN0b3J5OjI=" });
        
        done();
      });
    });

    it('should update the user with a new directory', function (done) {
      chai.request(url_wfgen_admin)
      .post('/')
      .send({
      'operation': 'updateUser',
        'args': {
          'input': {
            'id': 'VXNlcjoz',
            'directoryId': 'RGlyZWN0b3J5OjE='
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');

        expect(res.body.clientMutationId).to.be.null;
        expect(res.body.userId).to.not.be.null;

        userId = res.body.userId;

        done();
      });
    });

    it('should return the updated user with the new directory ', function (done) {
        if (userId === null) {
          this.skip();
        }
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
          'operation': 'getUser',
          'args': {
            'id': userId
          }
        })
        .end(function (err, res) {
          res.should.have.status(200);
          res.should.be.json;
  
          utils.validateUserProperties(res.body);
  
          expect(res.body).to.include({ id: userId, userName: 'john_smith', lastName: "Smith", directoryId: "RGlyZWN0b3J5OjE=" });
          
          done();
        });
    });

    it('should get an error when the directory is updated (requires a password) and the password is not set', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
        'operation': 'updateUser',
          'args': {
            'input': {
              'id': 'VXNlcjo0',
              'directoryId': 'RGlyZWN0b3J5OjM='
            }
          }
        })
        .end(function (err, res) {
            res.should.have.status(500);
            res.should.be.json;

            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The password is required');

            done();
        });
    });
});
