var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

var userId = null;
describe('UpdateViewerPreferences', function () {
  
  it('should update the viewer\'s language preference', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': 'en-CA'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');
        userId = res.body.userId;
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.userId.should.be.a('string');
        done();
      });
  });
  it('should return the viewer\'s updated language preference', function (done) {
    if (userId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('id');
        expect(res.body.id).to.be.equal(userId);
        userId = null;
        res.body.should.have.property('language');
        expect(res.body.language).to.be.equal('en-CA');
        done();
      });
  });
  it('should update the viewer\'s timezone preference', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'timeZoneId': 7
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');
        userId = res.body.userId;
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.userId.should.be.a('string');
        done();
      });
  });
  it('should return the viewer\'s updated timezone preference', function (done) {
    if (userId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('id');
        expect(res.body.id).to.be.equal(userId);
        userId = null;
        res.body.should.have.property('timeZoneId');
        expect(res.body.timeZoneId).to.be.equal(7);
        done();
      });
  });
  it('should update the viewer \'s language and timezone preferences', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': 'fr-CA',
            'timeZoneId': 8
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');
        userId = res.body.userId;
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.userId.should.be.a('string');
        done();
      });
  });
  it('should return the viewer\'s updated language and timezone preferences', function (done) {
    if (userId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('id');
        expect(res.body.id).to.be.equal(userId);
        userId = null;
        res.body.should.have.property('timeZoneId');
        expect(res.body.timeZoneId).to.be.equal(8);
        res.body.should.have.property('language');
        expect(res.body.language).to.be.equal('fr-CA');
        done();
      });
  });
  it('should get an error when the language value is invalid', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': 'es-CA',
            'timeZoneId': 9
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('Language is invalid:');
        done();
      });
  });
  it('should get an error when the timeZoneId value is invalid', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': 'en-CA',
            'timeZoneId': 84
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        res.body.should.have.property('error');
        res.body.error.should.be.a('string');
        expect(res.body.error).to.contain('Time zone id is invalid:');
        done();
      });
  });
  it('should update the viewer\'s language preference with a null value', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': null,
            'timeZoneId': 9
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');
        userId = res.body.userId;
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.userId.should.be.a('string');
        done();
      });
  });
  it('should return the viewer\'s updated language preference', function (done) {
    if (userId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('id');
        expect(res.body.id).to.be.equal(userId);
        userId = null;
        res.body.should.have.property('timeZoneId');
        expect(res.body.timeZoneId).to.be.equal(9);
        res.body.should.have.property('language');
        expect(res.body.language).to.be.equal(res.body.defaultLanguage);
        done();
      });
  });
  it('should update the viewer\'s timezone preference with a null value', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'updateViewerPreferences',
        'args': {
          'input': {
            'language': 'en-CA',
            'timeZoneId': null
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('clientMutationId');
        res.body.should.have.property('userId');
        userId = res.body.userId;
        expect(res.body.clientMutationId).to.be.a('null');
        res.body.userId.should.be.a('string');
        done();
      });
  });
  it('should return the viewer\'s updated timezone preference', function (done) {
    if (userId === null) {
      this.skip();
    }
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getUser',
        'args': {
          'id': userId
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        res.body.should.have.property('id');
        expect(res.body.id).to.be.equal(userId);
        userId = null;
        res.body.should.have.property('timeZoneId');
        expect(res.body.timeZoneId).to.be.equal(res.body.defaultTimeZoneId);
        res.body.should.have.property('language');
        expect(res.body.language).to.be.equal('en-CA');
        done();
      });
  });
});
