var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanAssignAction', function () {
    
    it('should return false when the action is already closed', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignAction',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when a standard user has the permission to assign the action', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignAction',
            'args' : {
                'id' : 'QWN0aW9uOjItLS0x',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false when a standard user does not have the permission to assign the action', function (done) {
        chai.request(url_user_manager)
        .post('/')
        .send({
            'operation': 'canAssignAction',
            'args' : {
                'id' : 'QWN0aW9uOjItLS0x',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when the process supervisor has the permission to re-assign the action', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canAssignAction',
            'args' : {
                'id' : 'QWN0aW9uOjU3LS0tMg=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false when the process supervisor does not have the permission to re-assign the action as a standard user', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canAssignAction',
            'args' : {
                'id' : 'QWN0aW9uOjU3LS0tMg==',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

});
