var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanAssignUserToAction', function () {

    it('should return false when the action is already closed', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0x',
                'assigneeId' : 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return false when a standard user does not have the permission to assign the action', function (done) {
        chai.request(url_user_manager)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjItLS0x',
                'assigneeId' : 'VXNlcjoy',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when the assignor has the permission and the assignee "mark_twain" is a member of the action participant', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjUtLS0x',
                'assigneeId' : 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false when the assignor has the permission and the assignee "evan_strong" is not a member of the action participant', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjUtLS0x',
                'assigneeId' : 'VXNlcjo2'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when it is a team action and the assignor and the assignee are the same person', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0y',
                'assigneeId' : 'VXNlcjox',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false when it is a team action and the assignor and the assignee are a different person', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canAssignUserToAction',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0y',
                'assigneeId' : 'VXNlcjo5',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

});