var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanCancelAction', function () {
    
    it('should return true if the action can be cancelled by an administrator', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjE2LS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false if the action does not have a default exception or an action cancellation', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjM4LS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true if the action can be cancelled by supervisors and the viewer is a supervisor', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjU3LS0tMg=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false if the action is closed', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjU3LS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return false if the action can be cancelled only by users and the viewer is a supervisor', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjYwLS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true if the action is assigned to the viewer and can be cancelled by users', function (done) {
        chai.request(url_user_standard)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjU2LS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false if the action is not assigned to the viewer', function (done) {
        chai.request(url_user_standard)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjQwLS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return false if the action can be cancelled only by supervisors and the viewer is a standard user', function (done) {
        chai.request(url_user_standard)
        .post('/')
        .send({
            'operation': 'canCancelAction',
            'args' : {
                'id' : 'QWN0aW9uOjU5LS0tMg=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });
});
