var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanCancelActionAssignment', function () {
    
    it('should return false when the action is already closed', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canCancelActionAssignment',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return false when the action is not assigned to someone', function (done) {
        chai.request(url_user_manager)
        .post('/')
        .send({
            'operation': 'canCancelActionAssignment',
            'args' : {
                'id' : 'QWN0aW9uOjItLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when the process supervisor has the permission to cancel the assignment of the action', function (done) {
        chai.request(url_user_supervisor)
        .post('/')
        .send({
            'operation': 'canCancelActionAssignment',
            'args' : {
                'id' : 'QWN0aW9uOjU3LS0tMg=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

    it('should return false when the action is automatically assigned to the requester by the system', function (done) {
        chai.request(url_user_standard)
        .post('/')
        .send({
            'operation': 'canCancelActionAssignment',
            'args' : {
                'id' : 'QWN0aW9uOjU4LS0tMQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.false;

            done();
        });
    });

    it('should return true when the user is assigned to the team action', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'canCancelActionAssignment',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0y',
                'input' : {
                    'isSuperUserMode' : false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            // Values validation
            expect(res.body).to.be.true;

            done();
        });
    });

});
