var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanCompleteAction', function () {
  it('should return true if the viewer can complete an action given the action ID', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjEtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(true);
        done();
      });
  });
  it('should return true if the viewer can complete an action whose activity instance ID is different from the activity ID', function (done) {
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjYzLS0tNA=='
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(true);
        done();
      });
  });
  it('should return false if the viewer is not a participant of the action', function (done) {
    chai.request(url_user_requester)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjQtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return true if the assignee is different than the current user and the action is in self service', function (done) {
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjctLS0y'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(true);
        done();
      });
  });
  it('should return false if the assignee is different than the current user and the action is not in self service', function (done) {
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjQtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return false if the action state is closed', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjYtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return an error if the activity instance cannot be found', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCompleteAction',
        'args': {
          'id': 'QWN0aW9uOjEwMDAtLS0x'
        }
      })
      .end(function (err, res) {
        res.should.have.status(500);
        res.should.be.json;
        expect(res.body.error).to.contain('The activity instance cannot be found.');
        done();
      });
  });
});
