var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('GetAction', function () {

    it('should get an action given its ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getAction',
            'args' : {
                'id' : 'QWN0aW9uOjEtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionProperties(res.body);

            // Values validation
            expect(res.body).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES' });

            done();
        });
    });
    it('should get the openedAt and assignedAt identical values for an action assigned by SYSTEM ', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getAction',
            'args' : {
                'id' : 'QWN0aW9uOjYtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionProperties(res.body);

            // Values validation
            var date = '2017-06-01T03:14:27.707Z';
            expect(res.body).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES', openedAt: date, assignedAt: date, assignorId: null });

            done();
        });
    });
    it('should get the openedAt and assignedAt different values for an action assigned by an user ', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getAction',
            'args' : {
                'id' : 'QWN0aW9uOjUtLS0x'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionProperties(res.body);

            // Values validation
            expect(res.body).to.include({ id:'QWN0aW9uOjUtLS0x', name: '1-INITIATES', openedAt: '2017-06-01T03:13:49.600Z', assignedAt: '2017-09-09T07:01:38.600Z', assignorId:'VXNlcjox' });

            done();
        });
    });
});
