var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetActions', function () {

    it('should get actions with as:ASSIGNEE filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'ASSIGNEE'
                },
                'page' : {
                    'size' : 60,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 57);
            
            // Values validation
            res.body.items.should.not.all.have.property('assigneeId', null);

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[56]).to.include({ id:'QWN0aW9uOjYzLS0tMw==', name: '3-UPDATES'});

            done();
        });
    });

    it('should get actions with as:ASSIGNOR filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'ASSIGNOR'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 5);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[4]).to.include({ id:'QWN0aW9uOjUtLS0x', name: '1-INITIATES'});

            done();
        });
    });

    it('should get actions with isSelfService:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 15);            

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES'});
            expect(res.body.items[14]).to.include({ id:'QWN0aW9uOjYzLS0tNA==', name: '4-VALIDATES'});

            done();
        });
    });

    it('should get actions with isSelfService:true and as:ASSIGNEE filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true,
                    'as': 'ASSIGNEE'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 14);            
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES'});
            expect(res.body.items[13]).to.include({ id:'QWN0aW9uOjYzLS0tMg==', name: '2-VALIDATES'});
             
            done();
        });
    });

    it('should get actions with as:SUPERVISOR filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'as': 'SUPERVISOR'
                },
                'page': {
                    'size': 80
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 79);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[78]).to.include({ id:'QWN0aW9uOjYzLS0tNA==', name: '4-VALIDATES'});
            
            done();
        });
    });

    it('should get actions without filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'page': {
                    'size': 80
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 79);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[78]).to.include({ id:'QWN0aW9uOjYzLS0tNA==', name: '4-VALIDATES'});
            
            done();
        });
    });

    it('should get actions with userAsMember:true arg', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'userAsMember': true,
                'page': {
                    'size': 70
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 64);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[63]).to.include({ id:'QWN0aW9uOjYzLS0tNA==', name: '4-VALIDATES'});
            
            done();
        });
    });

    it('should get actions by request ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestId': 'UmVxdWVzdDoyNg=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDoyNg==');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjI2LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions with status:CLOSED filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'status': 'CLOSED'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 20);

            // Values validation
            res.body.items.should.all.have.property('status', 'CLOSED');
            
            done();
        });
    });

    it('should get actions with assigneeId:"mark_twain" filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'assigneeId': 'VXNlcjoy'
                },
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 10);

            // Values validation
            res.body.items.should.all.have.property('assigneeId', 'VXNlcjoy');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjI2LS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[9]).to.include({ id:'QWN0aW9uOjM1LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get actions with processName:"TEST_SIMPLE_APPROVAL_LOOP" filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'processName': 'TEST_SIMPLE_APPROVAL_LOOP'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 19);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjE2LS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[18]).to.include({ id:'QWN0aW9uOjM1LS0tMQ==', name: '1-INITIATES'});
            
            done();
        });
    });

    it('should get all my actions in progress that are past due', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'status' : 'OPEN',
                    'as': 'ASSIGNEE',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 3);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjUxLS0tMg==', name: '2-VALIDATES'});
            expect(res.body.items[2]).to.include({ id:'QWN0aW9uOjU1LS0tMg==', name: '2-VALIDATES'});
            
            done();
        });
    });

    it('should get all my team actions that are or were past due', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isSelfService': true,
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 3);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjUxLS0tMg==', name: '2-VALIDATES'});
            expect(res.body.items[2]).to.include({ id:'QWN0aW9uOjU1LS0tMg==', name: '2-VALIDATES'});
            
            done();
        });
    });

    it('should get all actions by request ID that are or were past due', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'requestId': 'UmVxdWVzdDo1MQ==',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjUxLS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uOjUxLS0tMg==', name: '2-VALIDATES'});
            
            done();
        });
    });

    it('should get all actions that are or were past due', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 6);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjUxLS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[5]).to.include({ id:'QWN0aW9uOjU1LS0tMg==', name: '2-VALIDATES'});
            
            done();
        });
    });
    
    it('should get actions with as:ASSIGNOR and createdSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'createdSince': '2017-06-01T03:13:44.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:13:44.000Z'));
            
            done();
        });
    });

    it('should get actions with isSelfService:true and createdSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'isSelfService': true,
                    'createdSince': '2017-12-08T06:06:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 9);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-12-08T06:06:00.000Z'));
            
            done();
        });
    });

    it('should get actions with createdSince filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'createdSince': '2017-06-01T03:19:30.000Z'
                },
                'page' : {
                    'size' : 55,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 50);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:19:30.000Z'));
            
            done();
        });
    });
    
    it('should get an error when the filter createSince does not contain a valid DateTime', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'isSelfService': true,
                    'as': 'ASSIGNEE',
                    'createdSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The createdSince filter does not contain a valid DateTime.');
  
            done();
        });
    });

    it('should get actions with as:ASSIGNEE and assignedSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNEE',
                    'assignedSince': '2017-10-01T07:08:26.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 7);

            // Values validation
            res.body.items.map ( key => expect(key.assignedAt).to.be.above('2017-10-01T07:08:26.000Z'));
            
            done();
        });
    });

    it('should get actions with as:ASSIGNOR and assignedSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'assignedSince': '2017-09-10T07:08:26.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);
            
            // Values validation
            res.body.items.map ( key => expect(key.assignedAt).to.be.above('2017-09-10T07:08:26.000Z'));

            done();
        });
    });

    it('should get actions with as:SUPERVISOR and assignedSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'SUPERVISOR',
                    'assignedSince': '2017-09-01T03:13:40.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 11);
            
            // Values validation
            res.body.items.map ( key => expect(key.assignedAt).to.be.above('2017-09-01T03:13:40.000Z'));

            done();
        });
    });

    it('should get actions with assignedSince filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'assignedSince': '2017-11-01T03:13:40.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 9);
            
            // Values validation
            res.body.items.map ( key => expect(key.assignedAt).to.be.above('2017-11-01T03:13:40.000Z'));

            done();
        });
    });


    it('should get an error when the filter assignedSince does not contain a valid DateTime', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'assignedSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The assignedSince filter does not contain a valid DateTime.');
  
            done();
        });
    });

    it('should get actions with viewerAsMember filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
               'viewerAsMember': true,
                'page': {
                    'size': 100
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 79);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[78]).to.include({ id:'QWN0aW9uOjYzLS0tNA==', name: '4-VALIDATES'});

            done();
        });
    });

    it('should get an error when status:OPEN filter and subStatuses:[COMPLETED, RUNNING] filter are using together', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'status': 'OPEN',
                    'subStatuses': ['COMPLETED', 'RUNNING']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:OPEN filter cannot be used together with the subStatuses:[ABORTED or COMPLETED] filter.');
  
            done();
        });
    });

    it('should get an error when status:CLOSED filter and subStatuses:[NOT_RUNNING_NOT_STARTED] filter are using together', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'status': 'CLOSED',
                    'subStatuses': ['NOT_RUNNING_NOT_STARTED', 'ABORTED']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:CLOSED filter cannot be used together with the subStatuses:[RUNNING or NOT_RUNNING_NOT_STARTED] filter.');
  
            done();
        });
    });
    
    it('should get actions with subStatuses:[NOT_RUNNING_NOT_STARTED, RUNNING], isSelfService:true, as:ASSIGNEE and status:OPEN filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'isSelfService': true,
                    'as': 'ASSIGNEE',
                    'status': 'OPEN',
                    'subStatuses': ['NOT_RUNNING_NOT_STARTED', 'RUNNING']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 10);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES'});
            expect(res.body.items[9]).to.include({ id:'QWN0aW9uOjU5LS0tMg==', name: '2-VALIDATES'});

            done();
        });
    });

    it('should get actions with subStatuses:[NOT_RUNNING_NOT_STARTED] and as:ASSIGNOR filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'ASSIGNOR',
                    'subStatuses': ['NOT_RUNNING_NOT_STARTED']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 5);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjEtLS0x', name: '1-INITIATES'});
            expect(res.body.items[4]).to.include({ id:'QWN0aW9uOjUtLS0x', name: '1-INITIATES'});

            done();
        });
    });

    it('should get actions with subStatuses:[COMPLETED], status:CLOSED and as:SUPERVISOR filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'as': 'SUPERVISOR',
                    'status': 'CLOSED',
                    'subStatuses': ['COMPLETED']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 20);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES'});
            expect(res.body.items[19]).to.include({ id:'QWN0aW9uOjYzLS0tMw==', name: '3-UPDATES'});

            done();
        });
    });

    it('should get actions with subStatuses:[RUNNING, ABORTED] filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {                
                'filter':{
                    'subStatuses': ['RUNNING', 'ABORTED']
                },
                'page' : {
                    'size' : 40,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 38);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjExLS0tMQ==', name: '1-INITIATES'});
            expect(res.body.items[37]).to.include({ id:'QWN0aW9uOjYyLS0tMQ==', name: '1-INITIATES'});

            done();
        });
    });
    it('should get actions by request number', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo2');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES' , status:'CLOSED'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES' , status:'OPEN'});
            
            done();
        });
    });
    it('should get actions by request number filtered by status:CLOSED', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'status':'CLOSED'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo2');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES', status:'CLOSED'});
            
            done();
        });
    });
    it('should get actions by request number filtered by status:OPEN', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'status':'OPEN'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo2');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES', status:'OPEN'});
            
            done();
        });
    });
    it('should get actions by request number filtered by status:CLOSED and isVisibleInHistory:true', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'status':'CLOSED',
                    'isVisibleInHistory':true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo2');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES', status:'CLOSED'});
            
            done();
        });
    });
    it('should get actions by request number filtered by status:CLOSED and isVisibleInHistory:false', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 55,
                    'status':'CLOSED',
                    'isVisibleInHistory':false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 1);

            // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo1NQ==');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjU1LS0tMQ==', name: '1-INITIATES', status:'CLOSED'});
            
            done();
        });
    });
    it('should get actions by request number filtered by isVisibleInHistory:true', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'isVisibleInHistory':true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);

             // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo2');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES', status:'CLOSED'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uOjYtLS0y', name: '2-VALIDATES', status:'OPEN'});
            
            done();
        });
    });
    it('should get actions by request number filtered by isVisibleInHistory:false', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 55,
                    'isVisibleInHistory':false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 2);

             // Values validation
            res.body.items.should.all.have.property('requestId', 'UmVxdWVzdDo1NQ==');

            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjU1LS0tMQ==', name: '1-INITIATES', status:'CLOSED'});
            expect(res.body.items[1]).to.include({ id:'QWN0aW9uOjU1LS0tMg==', name: '2-VALIDATES', status:'OPEN'});
            
            done();
        });
    });
    it('should get actions with a local participant ID filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    'participantId':'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6Mg=='
                },
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 13);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES'});
            expect(res.body.items[12]).to.include({ id:'QWN0aW9uOjYzLS0tMw==', name: '3-UPDATES'});
            
            done();
        });
    });

    it('should get actions with a global participant ID filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter': {
                    "participantId":'R2xvYmFsUGFydGljaXBhbnQ6Mg=='
                },
                'page': {
                    'size': 50
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActionPageListProperties(res.body, 13);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'QWN0aW9uOjYtLS0x', name: '1-INITIATES'});
            expect(res.body.items[12]).to.include({ id:'QWN0aW9uOjYzLS0tMw==', name: '3-UPDATES'});
            
            done();
        });
    });
    it('should get an error if the status:OPEN filter is used together with the isVisibleInHistory:false filter', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'status':'OPEN',
                    'isVisibleInHistory':false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:OPEN filter cannot be used together with the isVisibleInHistory filter.');
  
            done();
        });
    });
    it('should get an error if the status:OPEN filter is used together with the isVisibleInHistory:true filter', function (done){
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActions',
            'args': {
                'filter':{
                    'requestNumber': 6,
                    'status':'OPEN',
                    'isVisibleInHistory':true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:OPEN filter cannot be used together with the isVisibleInHistory filter.');
  
            done();
        });
    });
});
