var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetActivities', function () {

    it('should get activities by processId without filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActivities',
            'args': {
                'processId': "UHJvY2Vzczox"
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActivityPageListProperties(res.body, 3);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aXZpdHk6MS0tLTE=', name: 'INITIATES', type: 'APPLICATION' });
            expect(res.body.items[1]).to.include({ id:'QWN0aXZpdHk6MS0tLTM=', name: 'UPDATES',  type: 'APPLICATION' });
            expect(res.body.items[2]).to.include({ id:'QWN0aXZpdHk6MS0tLTI=', name: 'VALIDATES',  type: 'APPLICATION' });

            done();
        });
    });

    it('should get activities by processId with type:APPLICATION filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActivities',
            'args': {
                'processId': "UHJvY2Vzczox",
                'filter':{
                    'type': 'APPLICATION'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActivityPageListProperties(res.body, 3);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aXZpdHk6MS0tLTE=', name: 'INITIATES', type: 'APPLICATION' });
            expect(res.body.items[1]).to.include({ id:'QWN0aXZpdHk6MS0tLTM=', name: 'UPDATES',  type: 'APPLICATION' });
            expect(res.body.items[2]).to.include({ id:'QWN0aXZpdHk6MS0tLTI=', name: 'VALIDATES',  type: 'APPLICATION' });

            done();
        });
    });

    it('should get activities by processId with isVisibleInHistory: false filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActivities',
            'args': {
                'processId': "UHJvY2Vzczox",
                'filter':{
                    'isVisibleInHistory': false
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActivityPageListProperties(res.body, 0);

            done();
        });
    });

    it('should get the activities\' isVisibleInHistory field', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getActivities',
            'args': {
                'processId': "UHJvY2Vzczo0"
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateActivityPageListProperties(res.body, 3);

            // Values validation 
            expect(res.body.items[0]).to.include({ id:'QWN0aXZpdHk6NC0tLTE=', name: 'INITIATES', type: 'APPLICATION', isVisibleInHistory:false});
            expect(res.body.items[1]).to.include({ id:'QWN0aXZpdHk6NC0tLTM=', name: 'UPDATES',  type: 'APPLICATION', isVisibleInHistory:true });
            expect(res.body.items[2]).to.include({ id:'QWN0aXZpdHk6NC0tLTI=', name: 'VALIDATES',  type: 'APPLICATION', isVisibleInHistory:true });

            done();
        });
    });
});
