var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('GetCategories', function () {
  it('should get some categories given their IDs', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getCategories',
        'args': {
          'ids': ['Q2F0ZWdvcnk6MQ==', 'Q2F0ZWdvcnk6Mg==']
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateCategoryPageListProperties(res.body, 2);

        res.body.items.should.all.have.property('userId', null);
        res.body.items.should.all.have.property('onBehalfOf', null);
        expect(res.body.items).to.have.deep.property('[0].id', 'Q2F0ZWdvcnk6MQ==');
        expect(res.body.items).to.have.deep.property('[1].id', 'Q2F0ZWdvcnk6Mg==');
        expect(res.body.items).to.have.deep.property('[0].name', 'FINANCES');
        expect(res.body.items).to.have.deep.property('[1].name', 'MARKETING');
        expect(res.body.items).to.have.deep.property('[0].description', 'FINANCES');
        expect(res.body.items).to.have.deep.property('[1].description', 'MARKETING');

        done();
      });
  });
  it('should get some categories given their names', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getCategories',
        'args': {
          'names': ['FINANCES', 'MARKETING']
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        
        utils.validateCategoryPageListProperties(res.body, 2);

        res.body.items.should.all.have.property('userId', null);
        res.body.items.should.all.have.property('onBehalfOf', null);
        expect(res.body.items).to.have.deep.property('[0].id', 'Q2F0ZWdvcnk6MQ==');
        expect(res.body.items).to.have.deep.property('[1].id', 'Q2F0ZWdvcnk6Mg==');
        expect(res.body.items).to.have.deep.property('[0].name', 'FINANCES');
        expect(res.body.items).to.have.deep.property('[1].name', 'MARKETING');
        expect(res.body.items).to.have.deep.property('[0].description', 'FINANCES');
        expect(res.body.items).to.have.deep.property('[1].description', 'MARKETING');
        done();
      });
  });
  it('should get the categories filtered by process ID', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getCategories',
        'args': {
          'filter': {
            'processId': 'UHJvY2Vzczox'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateCategoryPageListProperties(res.body, 1);

        res.body.items.should.all.have.property('userId', null);
        res.body.items.should.all.have.property('onBehalfOf', null);
        expect(res.body.items).to.have.deep.property('[0].id', 'Q2F0ZWdvcnk6MQ==');
        expect(res.body.items).to.have.deep.property('[0].name', 'FINANCES');
        expect(res.body.items).to.have.deep.property('[0].description', 'FINANCES');
        done();
      });
  });
  it('should get the categories using the nameContains filter', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getCategories',
        'args': {
          'filter': {
            'nameContains': 'MAR'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateCategoryPageListProperties(res.body, 1);

        res.body.items.should.all.have.property('userId', null);
        res.body.items.should.all.have.property('onBehalfOf', null);
        expect(res.body.items).to.have.deep.property('[0].id', 'Q2F0ZWdvcnk6Mg==');
        expect(res.body.items).to.have.deep.property('[0].name', 'MARKETING');
        expect(res.body.items).to.have.deep.property('[0].description', 'MARKETING');
        done();
      });
  });
  it('should get the categories using the descriptionContains filter', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getCategories',
        'args': {
          'filter': {
            'descriptionContains': 'MAR'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateCategoryPageListProperties(res.body, 1);

        res.body.items.should.all.have.property('userId', null);
        res.body.items.should.all.have.property('onBehalfOf', null);
        expect(res.body.items).to.have.deep.property('[0].id', 'Q2F0ZWdvcnk6Mg==');
        expect(res.body.items).to.have.deep.property('[0].name', 'MARKETING');
        expect(res.body.items).to.have.deep.property('[0].description', 'MARKETING');
        done();
      });
  });
});
