var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);
    
describe('GetDelegableProcessParticipants', function () {
    it('should get viewer\'s delegable process participants', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 7);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[6]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjYtLS0xNA==' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered by process ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'processId' : 'UHJvY2Vzczoy'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 3);

            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[2]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS02' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered by participant ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'participantId' : 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 1);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS01' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered by global participant ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'participantId' : 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 1);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS01' });

            done();
        });
    });

    it('should get viewer\'s delegable process participants filtered category ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'categoryId' : 'Q2F0ZWdvcnk6Mg=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 3);
            
            // Values validation
            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS02' });
            expect(res.body.items[2]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });

            done();
        });
    });

    it('should get mark_twain\'s delegable process participants', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            // Values validation
            expect(res.body.items).to.have.lengthOf(4);
            expect(res.body.totalCount).to.equal(4);

            utils.validateDelegableProcessParticipantPageListProperties(res.body, 4);

            expect(res.body.items[0]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjItLS00' });
            expect(res.body.items[3]).to.include({ id:'RGVsZWdhYmxlUHJvY2Vzc1BhcnRpY2lwYW50OjMtLS03' });

            done();
        });
    });
    it('should get an error if the participantId has not a GlobalParticipant or LocalParticipant type', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'participantId' : 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnRzOjU='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            // Values validation
            expect(res.body.error).to.contain('The ID is invalid. The expected type is GlobalParticipant or LocalProcessParticipant.');

            done();
        });
    });
    it('should get an error if the participantId doesn\'t contain a numeric value', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDelegableProcessParticipants',
            'args': {
                'userId': 'VXNlcjox',
                'filter' : {
                    'participantId' : 'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6YQ=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            // Values validation
            expect(res.body.error).to.contain('The ID must contain a valid numeric value.');

            done();
        });
    });
});
