var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('GetDirectory', function () {

    it('should get a directory given its ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDirectory',
            'args': {
                'id': "RGlyZWN0b3J5OjE="
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateDirectoryProperties(res.body);

            expect(res.body).to.include({ id:'RGlyZWN0b3J5OjE=', name: 'WORKFLOWGEN', description: "WorkflowGen", isDefault: true, isUserPasswordManaged: false });
            
            done();
        })
    });

    it('should get a directory given its name', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDirectory',
            'args': {
                'name': "DIR2"
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateDirectoryProperties(res.body);

            expect(res.body).to.include({ id:'RGlyZWN0b3J5OjI=', name: 'DIR2', description: "Directory 2", isDefault: false, isUserPasswordManaged: false });
            
            done();
        });
    });

    it('should get the viewer\'s directory with userAsMember:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getDirectory',
            'args': {
                'id': "RGlyZWN0b3J5OjE=",
                'userAsMember': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;
            
            utils.validateDirectoryProperties(res.body);

            expect(res.body).to.include({ id:'RGlyZWN0b3J5OjE=', name: 'WORKFLOWGEN', description: "WorkflowGen", isDefault: true, isUserPasswordManaged: false });
            
            done();
        });
    });

});