var chai = require('chai');
var chaiHttp = require('chai-http');
var things = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(things);

describe('GetFavorites', function () {
  it('should get all the favorites for an user', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getFavorites',
        'args': {
          'userId': 'VXNlcjox'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;

        utils.validateFavoritePageListProperties(res.body, 2);

        expect(res.body.items).to.have.deep.property('[0].id', 'RmF2b3JpdGU6MQ==');
        expect(res.body.items).to.have.deep.property('[1].id', 'RmF2b3JpdGU6Mg==');
        res.body.items.should.all.have.property('ownerId', 'VXNlcjox');
        expect(res.body.items).to.have.deep.property('[0].description', 'Test simple approval favorite process');
        expect(res.body.items).to.have.deep.property('[1].description', 'Requests favorite view');
        expect(res.body.items).to.have.deep.property('[0].type', 'PROCESS');
        expect(res.body.items).to.have.deep.property('[1].type', 'VIEW');
        expect(res.body.items).to.have.deep.property('[0].itemId', 'UHJvY2Vzczox');
        expect(res.body.items).to.have.deep.property('[1].itemId', 'VmlldzoxLS0tMg==');
        res.body.items.should.all.have.property('onBehalfOf', null);
        done();
      });
  });
  it('should get favorites for an user filtered by their type', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'getFavorites',
        'args': {
          'userId': 'VXNlcjox',
          'filter': {
            'type': 'PROCESS'
          }
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        
        utils.validateFavoritePageListProperties(res.body, 1);

        res.body.items.should.all.have.property('type', 'PROCESS');
        expect(res.body.items).to.have.deep.property('[0].id', 'RmF2b3JpdGU6MQ==');
        expect(res.body.items).to.have.deep.property('[0].ownerId', 'VXNlcjox');
        expect(res.body.items).to.have.deep.property('[0].description', 'Test simple approval favorite process');
        expect(res.body.items).to.have.deep.property('[0].itemId', 'UHJvY2Vzczox');
        expect(res.body.items).to.have.deep.property('[0].onBehalfOf', null);
        done();
      });
  });
});
