var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetGroups', function () {

    it('should get all the groups', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getGroups',
            'args': {
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateGroupPageListProperties(res.body, 7);

            expect(res.body.items[0]).to.include({ id:'R3JvdXA6MQ==', name: 'Group A', description: "Group A", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[6]).to.include({ id:'R3JvdXA6Nw==', name: 'Group F', description: "Group F", directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get all the viewer\'s groups with userAsMember:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getGroups',
            'args': {
                'userAsMember': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateGroupPageListProperties(res.body, 2);
            
            expect(res.body.items[0]).to.include({ id:'R3JvdXA6MQ==', name: 'Group A', description: "Group A", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[1]).to.include({ id:'R3JvdXA6Mw==', name: 'Group C', description: "Group C", directoryId: "RGlyZWN0b3J5OjE=" });

            done();
        });
    });

    it('should get groups given an array of systemIdentifier', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getGroups',
            'args': {
                'systemIdentifiers': ["S-1-5-21-1214440339-2077806209-725345543-553", "S-1-5-21-1214440339-2077806209-725345543-520"]
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateGroupPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'R3JvdXA6MQ==', name: 'Group A', description: "Group A", directoryId: "RGlyZWN0b3J5OjE=", systemIdentifier: "S-1-5-21-1214440339-2077806209-725345543-553" });
            expect(res.body.items[1]).to.include({ id:'R3JvdXA6NA==', name: 'Group D', description: "Group D", directoryId: "RGlyZWN0b3J5OjI=", systemIdentifier: "S-1-5-21-1214440339-2077806209-725345543-520" });

            done();
        });
    });

});
