var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetProcessParticipants', function () {
    
    it('should get process participants by process ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getProcessParticipants',
            'args': {
                'processId': 'UHJvY2Vzczoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateParticipantPageListProperties(res.body, 3);

            // Values validation
            res.body.items.should.not.all.have.property('procesId', null);

            expect(res.body.items[0]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NQ==', name: 'LEVEL_1'});
            expect(res.body.items[2]).to.include({ id:'TG9jYWxQcm9jZXNzUGFydGljaXBhbnQ6NA==', name: 'REQUESTER'});

            done();
        });
    });

    it('should get process participants by process ID with isGlobal:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getProcessParticipants',
            'args': {
                'processId': 'UHJvY2Vzczoy',
                'filter' : {
                    'isGlobal' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateParticipantPageListProperties(res.body, 0);

            done();
        });
    });    
});
