var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('CanCreateRequest', function () {
  it('should return true if the viewer can create a request given the process ID', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCreateRequest',
        'args': {
          'id': 'UHJvY2Vzczox'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(true);
        done();
      });
  });
  it('should return false if the viewer is not a member of the requester participant of the process', function (done) {
    chai.request(url_user_manager)
      .post('/')
      .send({
        'operation': 'canCreateRequest',
        'args': {
          'id': 'UHJvY2Vzczox'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return false if the process state is archived', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCreateRequest',
        'args': {
          'id': 'UHJvY2Vzczo1'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return false if the process state is under construction', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCreateRequest',
        'args': {
          'id': 'UHJvY2Vzczo3'
        }
      })
      .end(function (err, res) {
        res.should.have.status(200);
        res.should.be.json;
        expect(res.body).to.equal(false);
        done();
      });
  });
  it('should return an error if no process ID is provided', function (done) {
    chai.request(url_wfgen_admin)
      .post('/')
      .send({
        'operation': 'canCreateRequest',
        'args': {
        }
      })
      .end(function (err, res) {
        res.should.have.status(400);
        res.should.be.json;
        expect(res.body.error).to.contain('A process ID is required.');
        done();
      });
  });
});
