var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
var validUrl = require('valid-url');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetLaunchableProcesses', function () {
    
    it('should get viewer\'s launchable processes', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 6);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczo4', name: 'SIMPLE_APPROVAL_TEST_EXCEPTION'});
            expect(res.body.items[5]).to.include({ id:'UHJvY2Vzczo2', name: 'TEST_SIMPLE_REQUEST_GLOBAL'});

            done();
        });
    });

    it('should get viewer\'s launchable processes filtered by categoryName', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjox',
                'filter' : {
                    'categoryName' : 'FINANCES'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 1);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczox', name: 'TEST_SIMPLE_APPROVAL'});

            done();
        });
    });

    it('should get viewer\'s launchable processes filtered by categoryId', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjox',
                'filter' : {
                    'categoryId' : 'Q2F0ZWdvcnk6MQ=='
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 1);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczox', name: 'TEST_SIMPLE_APPROVAL'});

            done();
        });
    });

    it('should get mark_twain\'s launchable processes', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjoy'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 2);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczoy', name: 'TEST_2_LEVELS_APPROVAL'});
            expect(res.body.items[1]).to.include({ id:'UHJvY2Vzczoz', name: 'TEST_SIMPLE_APPROVAL_LOOP'});

            done();
        });
    });

    it('should viewer\'s launchable processes\' graphical view urls', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getLaunchableProcesses',
            'args' : {
                'userId' : 'VXNlcjox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 6);

            // Values validation 
            expect(res.body.items[1]).to.include({ 
                id:'UHJvY2Vzczoy'
            });
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewUrl)).to.not.be.undefined
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewPortalUrl)).to.not.be.undefined
            expect(res.body.items[5]).to.include({ 
                id:'UHJvY2Vzczo2'
            });
            expect(validUrl.isUri(res.body.items[4].graphicalViewUrl)).to.not.be.undefined
            expect(validUrl.isUri(res.body.items[4].graphicalViewPortalUrl)).to.not.be.undefined

            done();
        });
    });
});