var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
var validUrl = require('valid-url');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetProcesses', function () {
    
    it('should get all processes without filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getProcesses'
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 8);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczo4', name: 'SIMPLE_APPROVAL_TEST_EXCEPTION'});
            expect(res.body.items[7]).to.include({ id:'UHJvY2Vzczo2', name: 'TEST_SIMPLE_REQUEST_GLOBAL'});

            done();
        });
    });

    it('should get processes with state:TEST filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getProcesses',
            'args': {
                'filter':{
                    'state': 'TEST'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 3);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UHJvY2Vzczo4', name: 'SIMPLE_APPROVAL_TEST_EXCEPTION'});
            expect(res.body.items[2]).to.include({ id:'UHJvY2Vzczo0', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME'});

            done();
        });
    });

    it('should verify processes\' graphical view urls', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getProcesses'
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateProcessPageListProperties(res.body, 8);
            
            // Values validation 
            expect(res.body.items[1]).to.include({ 
                id:'UHJvY2Vzczo3'
            });
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewUrl)).to.not.be.undefined
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewPortalUrl)).to.not.be.undefined
            expect(res.body.items[7]).to.include({ 
                id:'UHJvY2Vzczo2'
            });
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewUrl)).to.not.be.undefined
            expect(validUrl.isWebUri(res.body.items[0].graphicalViewPortalUrl)).to.not.be.undefined

            done();
        });
    });
});
