var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
chai.use(chaiHttp);

describe('HasGraphicalViewAccess', function(){
    it('should return true if the viewer has graphical view access', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'hasGraphicalViewAccess',
            'args' : {
                'id' : 'UHJvY2Vzczox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            expect(res.body).to.equal(true);
            done();
        });
    });
    it('should return false if the viewer don\'t have graphical view access', function (done) {
        chai.request(url_user_requester)
        .post('/')
        .send({
            'operation': 'hasGraphicalViewAccess',
            'args' : {
                'id' : 'UHJvY2Vzczox'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            expect(res.body).to.equal(false);
            done();
        });
    });
    it('should return an error if no process ID is provided', function (done) {
        chai.request(url_user_requester)
        .post('/')
        .send({
            'operation': 'hasGraphicalViewAccess',
            'args' : {
               
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            expect(res.body.error).to.equal('A process ID is required.');
            done();
        });
    });
});
