var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('GetRequest', function(){

    it('should get a request given its ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequest',
            'args' : {
                'id' : 'UmVxdWVzdDo5'
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestProperties(res.body);

            expect(res.body).to.include({ id:'UmVxdWVzdDo5', name: 'TEST_SIMPLE_APPROVAL #9' });

            done();
        });
    });

    it('should get a request given its number', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequest',
            'args' : {
                'number' : 7
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;
            
            utils.validateRequestProperties(res.body);

            expect(res.body).to.include({ id:'UmVxdWVzdDo3', name: 'TEST_SIMPLE_APPROVAL #7' });

            done();
        });
    });
    it('should get an error if the request ID has not a valid type', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequest',
            'args' : {
                'id' : 'UmVxdWVzdHM6OQ=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            expect(res.body.error).to.contain("The ID is invalid. The expected type is Request, but got Requests.");

            done();
        });
    });
    it('should get an error if the request doesn\'t contain a valid numeric value', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequest',
            'args' : {
                'id' : 'UmVxdWVzdDph'
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            expect(res.body.error).to.contain("The ID must contain a valid numeric value.");

            done();
        });
    });
    it('should get an error if the request ID is invalid', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequest',
            'args' : {
                'id' : 'UmVxdWVzdA=='
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;

            expect(res.body.error).to.contain("The ID is invalid.");

            done();
        });
    });
});
