var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetRequests', function () {

    it('should get all requests without filter and with page size', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'page' : {
                    'size' : 65,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 63);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[62]).to.include({ id:'UmVxdWVzdDo2Mw==', name: 'TEST_SIMPLE_APPROVAL #63'});

            done();
        });
    });

    it('should get all requests with isTest:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'page' : {
                    'size' : 35,
                    'number' : 1
                },
                'filter' : {
                    'isTest' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 34);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo2', name: 'TEST_SIMPLE_APPROVAL #6'});
            expect(res.body.items[33]).to.include({ id:'UmVxdWVzdDo2Mw==', name: 'TEST_SIMPLE_APPROVAL #63'});

            done();
        });
    });

    it('should get all requests with as:REQUESTER filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as' : 'REQUESTER'
                },
                'page' : {
                    'size' : 55,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 48);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[47]).to.include({ id:'UmVxdWVzdDo2Mw==', name: 'TEST_SIMPLE_APPROVAL #63'});

            done();
        });
    });

    it('should get all requests with userAsMember arg', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userAsMember' : true,
                'page' : {
                    'size' : 55,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 49);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[48]).to.include({ id:'UmVxdWVzdDo2Mw==', name: 'TEST_SIMPLE_APPROVAL #63'});
            
            done();
        });
    });

    it('should get all requests by requesterID filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'requesterId' : 'VXNlcjoy'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 14);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoyNw==', name: 'TEST_SIMPLE_APPROVAL_LOOP #27'});
            expect(res.body.items[13]).to.include({ id:'UmVxdWVzdDo0MA==', name: 'TEST_2_LEVELS_APPROVAL #40'});

            done();
        });
    });

    it('should get all requests closed', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'status' : 'CLOSED'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0MQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #41'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdDo2MQ==', name: 'SIMPLE_APPROVAL_TEST_EXCEPTION #61'});

            done();
        });
    });

    it('should get all requests by processName filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'processName' : 'TEST_SIMPLE_APPROVAL_LOOP'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 19);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoxNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #16'});
            expect(res.body.items[18]).to.include({ id:'UmVxdWVzdDozNQ==', name: 'TEST_SIMPLE_APPROVAL_LOOP #35'});

            done();
        });
    });

    it('should get all requests by processName and processVersion filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'processName' : 'TEST_SIMPLE_APPROVAL_LEAD_TIME',
                    'processVersion': 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 15);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0MQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #41'});
            expect(res.body.items[14]).to.include({ id:'UmVxdWVzdDo1NQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #55'});

            done();
        });
    });

    it('should get all requests from user "mark_twain"', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userId' : 'VXNlcjoy',
                'userAsMember' : true
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 15);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoyNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #26'});
            expect(res.body.items[14]).to.include({ id:'UmVxdWVzdDo0MA==', name: 'TEST_2_LEVELS_APPROVAL #40'});

            done();
        });
    });

    it('should get all requests from user "mark_twain" and viewerAsMember', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'userId' : 'VXNlcjoy',
                'userAsMember' : true,
                'viewerAsMember' : true
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 1);

             expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDoyNg==', name: 'TEST_SIMPLE_APPROVAL_LOOP #26' });

            done();
        });
    });

    it('should get all requests that are or were past due.', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0Mg==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #42'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdDo0OQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #49'});

            done();
        });
    });

    it('should get all requests in progress which I am the requester and that are past due.', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as': 'REQUESTER',
                    'status' : 'OPEN',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0Mg==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #42'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdDo0OQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #49'});

            done();
        });
    });

    it('should get all requests in progress which I supervise that are not past due.', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args' : {
                'filter' : {
                    'as' : 'SUPERVISOR',
                    'status' : 'OPEN',
                    'isDueDateExceeded' : true
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0Mg==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #42'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdDo0OQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #49'});

            done();
        });
    });

    it('should get requests with as:REQUESTER and createdSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'as': 'REQUESTER',
                    'createdSince': '2017-06-01T03:17:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 28);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:17:00.000Z'));
            
            done();
        });
    });

    it('should get requests with as:SUPERVISOR and createdSince filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'as': 'SUPERVISOR',
                    'createdSince': '2017-06-01T03:20:00.000Z'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 30);
            
            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:20:00.000Z'));
            
            done();
        });
    });

    it('should get requests with createdSince filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'createdSince': '2017-06-01T03:17:00.000Z',
                },
                'page' : {
                    'size' : 55,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 42);

            // Values validation
            res.body.items.map ( key => expect(key.openedAt).to.be.above('2017-06-01T03:17:00.000Z'));
            
            done();
        });
    });
    
    it('should get an error when the filter createSince does not contain a valid DateTime', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'createdSince': 'invalid date'
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The createdSince filter does not contain a valid DateTime.');
  
            done();
        });
    });

    it('should get an error when status:OPEN filter and subStatuses:[COMPLETED] filter are using together', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'status': 'OPEN',
                    'subStatuses': ['COMPLETED', 'RUNNING']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:OPEN filter cannot be used together with the subStatuses:[ABORTED or COMPLETED] filter.');
  
            done();
        });
    });

    it('should get an error when status:CLOSED filter and subStatuses:[NOT_RUNNING_NOT_STARTED] filter are using together', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'status': 'CLOSED',
                    'subStatuses': ['NOT_RUNNING_NOT_STARTED', 'ABORTED']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(400);
            res.should.be.json;
    
            res.body.should.have.property('error');
            expect(res.body.error).to.contain('The status:CLOSED filter cannot be used together with the subStatuses:[RUNNING or NOT_RUNNING_NOT_STARTED] filter.');
  
            done();
        });
    });

    it('should get requests with as:REQUESTER, status:CLOSED and subStatuses:[COMPLETED] filters', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'as': 'REQUESTER',
                    'status': 'CLOSED',
                    'subStatuses': ['COMPLETED']
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 4);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDo0MQ==', name: 'TEST_SIMPLE_APPROVAL_LEAD_TIME #41'});
            expect(res.body.items[3]).to.include({ id:'UmVxdWVzdDo2MQ==', name: 'SIMPLE_APPROVAL_TEST_EXCEPTION #61'});

            done();
        });
    });

    it('should get requests with subStatuses:[RUNNING, ABORTED] filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getRequests',
            'args': {                
                'filter':{
                    'subStatuses': ['RUNNING', 'ABORTED']
                },
                'page' : {
                    'size' : 60,
                    'number' : 1
                }
            }
        })
        .end(function (err, res) {

            res.should.have.status(200);
            res.should.be.json;

            utils.validateRequestPageListProperties(res.body, 59);
            
            // Values validation 
            expect(res.body.items[0]).to.include({ id:'UmVxdWVzdDox', name: 'TEST_2_LEVELS_APPROVAL #1'});
            expect(res.body.items[58]).to.include({ id:'UmVxdWVzdDo2Mw==', name: 'TEST_SIMPLE_APPROVAL #63'});

            done();
        });
    });
});
