var chai = require('chai');
var chaiHttp = require('chai-http');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);

describe('GetUser', function () {

    it('should get an user given its ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUser',
            'args': {
                'id': "VXNlcjo1"
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserProperties(res.body);

            expect(res.body).to.include({ id:'VXNlcjo1', userName: 'jane_doe', lastName: "Doe", firstName: "Jane", email: "jane.doe@company.com", directoryId: "RGlyZWN0b3J5OjI=" });
            
            done();
        })
    });

    it('should get an user given its systemIdentifier', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUser',
            'args': {
                'systemIdentifier': "54D382A4-2050-4C03-94D1-E769F1D15682"
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserProperties(res.body);

            expect(res.body).to.include({ id:'VXNlcjoz', userName: 'john_smith', lastName: "Smith", firstName: "John", email: "johnsmith@test.com", directoryId: "RGlyZWN0b3J5OjE=", systemIdentifier: "54D382A4-2050-4C03-94D1-E769F1D15682" });
            
            done();
        })
    });

});