var chai = require('chai');
var chaiHttp = require('chai-http');
var chaiThings = require('chai-things');
var expect = require('chai').expect;
var should = chai.should();
var utils = require('../../utils');
chai.use(chaiHttp);
chai.use(chaiThings);

describe('GetUsers', function () {

    it('should get all users', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 11);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo2', userName: 'evan_strong', lastName: "Strong", firstName: "Evan", email: "evan.strong@company.com", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[10]).to.include({ id:'VXNlcjox', userName: 'wfgen_admin', lastName: "Administrator", firstName: "WorkflowGen", email: null, directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users with isActive:true filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'filter': {
                    'isActive': true
                }
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 11);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo2', userName: 'evan_strong', lastName: "Strong", firstName: "Evan", email: "evan.strong@company.com", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[10]).to.include({ id:'VXNlcjox', userName: 'wfgen_admin', lastName: "Administrator", firstName: "WorkflowGen", email: null, directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users with nameContains:\'doe\' filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'filter': {
                    'nameContains': 'doe'
                }
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo1', userName: 'jane_doe', lastName: "Doe", firstName: "Jane", email: "jane.doe@company.com", directoryId: "RGlyZWN0b3J5OjI=" });
            expect(res.body.items[1]).to.include({ id:'VXNlcjo0', userName: 'john_doe', lastName: "Doe", firstName: "John", email: "john.doe@company.com", directoryId: "RGlyZWN0b3J5OjI=" });
            
            done();
        });
    });

    it('should get users with isArchived:false filter', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'filter': {
                    'isArchived': false
                }
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 11);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo2', userName: 'evan_strong', lastName: "Strong", firstName: "Evan", email: "evan.strong@company.com", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[10]).to.include({ id:'VXNlcjox', userName: 'wfgen_admin', lastName: "Administrator", firstName: "WorkflowGen", email: null, directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users given an array of ID', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'ids': ["VXNlcjoz", "VXNlcjo0"],
                'isBatch': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo0', userName: 'john_doe', lastName: "Doe", firstName: "John", email: "john.doe@company.com", directoryId: "RGlyZWN0b3J5OjI=" });
            expect(res.body.items[1]).to.include({ id:'VXNlcjoz', userName: 'john_smith', lastName: "Smith", firstName: "John", email: "johnsmith@test.com", directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users given an array of userName', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'userNames': ["john_doe", "john_smith", "mark_twain"],
                'isBatch': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 3);

            expect(res.body.items[0]).to.include({ id:'VXNlcjo0', userName: 'john_doe', lastName: "Doe", firstName: "John", email: "john.doe@company.com", directoryId: "RGlyZWN0b3J5OjI=" });
            expect(res.body.items[1]).to.include({ id:'VXNlcjoz', userName: 'john_smith', lastName: "Smith", firstName: "John", email: "johnsmith@test.com", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[2]).to.include({ id:'VXNlcjoy', userName: 'mark_twain', lastName: "Twain", firstName: "Mark", email: "marktwain@test.com", directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users given an array of systemIdentifier', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'systemIdentifiers': ["54D382A4-2050-4C03-94D1-E769F1D15682", "2819C223-7F76-453A-919D-413861904646"],
                'isBatch': true
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'VXNlcjoz', userName: 'john_smith', lastName: "Smith", firstName: "John", email: "johnsmith@test.com", directoryId: "RGlyZWN0b3J5OjE=" });
            expect(res.body.items[1]).to.include({ id:'VXNlcjoy', userName: 'mark_twain', lastName: "Twain", firstName: "Mark", email: "marktwain@test.com", directoryId: "RGlyZWN0b3J5OjE=" });
            
            done();
        });
    });

    it('should get users given a groupId', function (done) {
        chai.request(url_wfgen_admin)
        .post('/')
        .send({
            'operation': 'getUsers',
            'args': {
                'groupId': 'R3JvdXA6Mw=='
            }
        })
        .end(function (err, res) {
            res.should.have.status(200);
            res.should.be.json;

            utils.validateUserPageListProperties(res.body, 2);

            expect(res.body.items[0]).to.include({ id:'VXNlcjoz', userName: 'john_smith', lastName: "Smith", firstName: "John", email: "johnsmith@test.com", directoryId: "RGlyZWN0b3J5OjE=", systemIdentifier: "54D382A4-2050-4C03-94D1-E769F1D15682" });
            expect(res.body.items[1]).to.include({ id:'VXNlcjox', userName: 'wfgen_admin', lastName: "Administrator", firstName: "WorkflowGen", email: null, directoryId: "RGlyZWN0b3J5OjE=", systemIdentifier: null });
            
            done();
        });
    });

});
