const sql = require('seriate');
const scripts = require('./utils');

// Create the database
var createDatabaseAndLogin = function (databaseName, loginName, loginPassword) {
  return new Promise(function (resolve, reject) {
    console.log('Creating the database and login account');
    sql.getPlainContext('Master-connection')
      .step('createDatabase', {
        query: 'CREATE DATABASE ' + databaseName
      })
      .step('createLogin', {
        query: 'CREATE LOGIN ' + loginName + " WITH PASSWORD = '" + loginPassword + "'"
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Creating the schema and user
var createSchemaAndUser = function (username) {
  return new Promise(function (resolve, reject) {
    console.log('Creating the database user');
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('createSchema', {
        query: 'CREATE SCHEMA ' + username
      })
      .step('createUser', {
        query: 'CREATE USER ' + username + ' FOR LOGIN ' + username + ' WITH DEFAULT_SCHEMA= ' + username
      })
      .step('AddDataReaderRole', {
        procedure: 'sp_addrolemember',
        params: {
          rolename: {
            type: sql.NVARCHAR,
            val: 'db_datareader'
          },
          membername: {
            type: sql.NVARCHAR,
            val: username
          }
        }
      })
      .step('AddDataWriterRole', {
        procedure: 'sp_addrolemember',
        params: {
          rolename: {
            type: sql.NVARCHAR,
            val: 'db_datawriter'
          },
          membername: {
            type: sql.NVARCHAR,
            val: username
          }
        }
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Create the tables, primary keys, foreign keys and indexes
var createTables = function () {
  return new Promise(function (resolve, reject) {
    console.log('Creating the database schema');
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('runScripts', {
        query: sql.fromFile(__dirname + '/scripts/Create_WFG-V7-0_SQL_Tables.sql')
      })
      .step('createPKeys', {
        query: sql.fromFile(__dirname + '/scripts/Create_WFG-V7-0_SQL_PKeys.sql')
      })
      .step('createFKeys', {
        query: sql.fromFile(__dirname + '/scripts/Create_WFG-V7-0_SQL_FKeys.sql')
      })
      .step('createIndexes', {
        query: sql.fromFile(__dirname + '/scripts/Create_WFG-V7-0_SQL_Indexes.sql')
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Create the triggers
var createTriggers = function () {
  console.log('Creating the database triggers');
  return scripts.splitTriggersScript(__dirname + '/scripts/Create_WFG-V7-0_SQL_Triggers.sql')
    .then(function (triggersArray) {
      return forEachPromise(triggersArray, createTrigger)
        .then(() => {
        });
    });
};

function createTrigger (item) {
  return new Promise((resolve, reject) => {
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('createTriggers', {
        query: item
      })
      .end(function () {
        resolve();
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
}

// Iterate the promises array
function forEachPromise (items, fn) {
  return items.reduce(function (promise, item) {
    return promise.then(function () {
      return fn(item);
    });
  }, Promise.resolve());
}

// Delete the test data set
var deleteRecords = function () {
  console.log('Deleting the database records');
  return new Promise(function (resolve, reject) {
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('deleteRecords', {
        query: sql.fromFile(__dirname + '/scripts/deletedataset.sql')
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Insert the constants
var insertConst = function () {
  console.log('Inserting the constants');
  return new Promise(function (resolve, reject) {
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('insertConst', {
        query: sql.fromFile(__dirname + '/scripts/Create_WFG-V7-0_SQL_Const.sql')
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Insert the test data set
var insertData = function () {
  return new Promise(function (resolve, reject) {
    console.log('Inserting the data');
    sql.getPlainContext('WFGEN_TEST-connection')
      .step('insertData', {
        query: sql.fromFile(__dirname + '/scripts/insertdataset.sql')
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Drop the database
var dropDatabase = function (databaseName) {
  return new Promise(function (resolve, reject) {
    console.log('Dropping the database');
    sql.getPlainContext('Master-connection')
      .step('killDatabaseProcesses', {
        query: sql.fromFile(__dirname + '/scripts/killdatabaseprocesses.sql'),
        params: {
          databaseName: {
            type: sql.NVARCHAR,
            val: databaseName
          }
        }
      })
      .step('dropDatabase', {
        query: 'DROP DATABASE ' + databaseName
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

// Drop the login
var dropLogin = function (login) {
  return new Promise(function (resolve, reject) {
    console.log('Dropping the login account');
    sql.getPlainContext('Master-connection')
      .step('dropLogin', {
        procedure: 'sp_droplogin',
        params: {
          loginame: {
            type: sql.NVARCHAR,
            val: login
          }
        }
      })
      .end(function (result) {
        resolve(result);
      })
      .error(function (err) {
        console.log(err);
        reject(err);
      });
  });
};

module.exports = {
  createDatabaseAndLogin,
  createSchemaAndUser,
  createTables,
  createTriggers,
  insertConst,
  deleteRecords,
  dropDatabase,
  dropLogin,
insertData};
