var expect = require('chai').expect;

var utils = {
  validateObjectProperties(objectPageList, totalCount, arrayAttributes) {

    // Properties
    objectPageList.should.have.property('totalCount');
    objectPageList.should.have.property('items');

    // Count
    expect(objectPageList.items).to.have.lengthOf(totalCount);
    expect(objectPageList.totalCount).to.equal(totalCount);

    // Array validation
    objectPageList.items.should.be.a('array');

    // items validation
    arrayAttributes.map(key => objectPageList.items.should.all.have.property(key));
  },

  getActionAttributes() {
    return ['id', 'name', 'description', 'number', 'requestId', 'childRequestId', 'status', 'subStatus', 'activityId',
      'initiatorId', 'assigneeId', 'delegatedAssigneeId', 'assignorId', 'abortedBy', 'limit', 'openedAt', 'startedAt',
      'updatedAt', 'closedAt', 'assignedAt', 'followUpFormUrl', 'followUpFormPortalUrl', 'launchUrl', 'launchPortalUrl',
      'userId', 'onBehalfOf'];
  },

  validateActionProperties(action) {
    this.getActionAttributes().map(key => action.should.have.property(key));
  },

  validateActionPageListProperties(actionPageList, totalCount) {
    this.validateObjectProperties(actionPageList, totalCount, this.getActionAttributes());
  },

  getActionDataAttributes() {
    return ['id', 'name', 'description', 'type', 'textValue', 'numericValue', 'dateTimeValue', 'fileValue',
      'hasValue', 'actionId', 'userId', 'onBehalfOf'];
  },

  validateActionDataProperties(actionData) {
    this.getActionDataAttributes().map(key => actionData.should.have.property(key));
  },

  validateActionDataPageListProperties(actionDataPageList, totalCount) {
    this.validateObjectProperties(actionDataPageList, totalCount, this.getActionDataAttributes());
  },

  getActivityAttributes() {
    return ['id', 'name', 'description', 'type', 'processId', 'participantId', 'applicationId', 'help', 'isVisibleInHistory', 'userId', 'onBehalfOf'];
  },

  validateActivityProperties(activity) {
    this.getActivityAttributes().map(key => activity.should.have.property(key));
  },

  validateActivityPageListProperties(activityPageList, totalCount) {
    this.validateObjectProperties(activityPageList, totalCount, this.getActivityAttributes());
  },

  getApplicationAttributes() {
    return ['id', 'name', 'description', 'type', 'url', 'wsdl', 'username', 'password', 'assemblyName', 'assemblyClassName',
      'method', 'contentType', 'contextFormat', 'isSchemaDefinitionEmbedded', 'isDefault', 'isActive'];
  },

  validateApplicationProperties(application) {
    this.getApplicationAttributes().map(key => application.should.have.property(key));
  },

  validateApplicationPageListProperties(applicationPageList, totalCount) {
    this.validateObjectProperties(applicationPageList, totalCount, this.getApplicationAttributes());
  },
  
  getApplicationParameterAttributes() {
    return ['id', 'name', 'description', 'dataType', 'direction', 'isRequired', 'isDefault'];
  },

  validateApplicationParameterProperties(applicationParameter) {
    this.getApplicationParameterAttributes().map(key => applicationParameter.should.have.property(key));
  },

  validateApplicationParametersPageListProperties(applicationParametersPageList, totalCount) {
    this.validateObjectProperties(applicationParametersPageList, totalCount, this.getApplicationParameterAttributes());
  },

  getCategoryAttributes() {
    return ['id', 'name', 'description', 'userId', 'onBehalfOf'];
  },

  validateCategoryProperties(category) {
    this.getCategoryAttributes().map(key => category.should.have.property(key));
  },

  validateCategoryPageListProperties(categoryPageList, totalCount) {
    this.validateObjectProperties(categoryPageList, totalCount, this.getCategoryAttributes());
  },

  getCommentAttributes() {
    return ['id', 'message', 'postedAt', 'authorId', 'subjectId', 'onBehalfOf', 'userId'];
  },

  validateCommentProperties(comment) {
    this.getCommentAttributes().map(key => comment.should.have.property(key));
  },

  validateCommentPageListProperties(commentPageList, totalCount) {
    this.validateObjectProperties(commentPageList, totalCount, this.getCommentAttributes());
  },

  getDelegationAttributes() {
    return ['id', 'delegatorId', 'delegateeId', 'start', 'end', 'processId', 'participantId'];
  },

  validateDelegationProperties(delegation) {
    this.getDelegationAttributes().map(key => delegation.should.have.property(key));
  },

  validateDelegationPageListProperties(delegationPageList, totalCount) {
    this.validateObjectProperties(delegationPageList, totalCount, this.getDelegationAttributes());
  },

  getDelegableProcessParticipantAttributes() {
    return ['id', 'processId', 'participantId'];
  },

  validateDelegableProcessParticipantProperties(delegableProcessParticipant) {
    this.getDelegableProcessParticipantAttributes().map(key => delegableProcessParticipant.should.have.property(key));
  },

  validateDelegableProcessParticipantPageListProperties(delegableProcessParticipantPageList, totalCount) {
    this.validateObjectProperties(delegableProcessParticipantPageList, totalCount, this.getDelegableProcessParticipantAttributes());
  },

  getDelegatorAttributes() {
    return ['id', 'delegatorId', 'delegateeId', 'start', 'end', 'processId', 'participantId'];
  },

  validateDelegatorProperties(delegator) {
    this.getDelegatorAttributes().map(key => delegator.should.have.property(key));
  },

  validateDelegatorPageListProperties(delegatorPageList, totalCount) {
    this.validateObjectProperties(delegatorPageList, totalCount, this.getDelegatorAttributes());
  },

  getDirectoryAttributes() {
    return ['id', 'name', 'description', 'isDefault', 'isUserPasswordManaged'];
  },

  validateDirectoryProperties(directory) {
    this.getDirectoryAttributes().map(key => directory.should.have.property(key));
  },

  validateDirectoryPageListProperties(directoryPageList, totalCount) {
    this.validateObjectProperties(directoryPageList, totalCount, this.getDirectoryAttributes());
  },

  getFavoriteAttributes() {
    return ['id', 'ownerId', 'description', 'type', 'itemId', 'onBehalfOf'];
  },

  validateFavoriteProperties(favorite) {
    this.getFavoriteAttributes().map(key => favorite.should.have.property(key));
  },

  validateFavoritePageListProperties(favoritePageList, totalCount) {
    this.validateObjectProperties(favoritePageList, totalCount, this.getFavoriteAttributes());
  },

  getGroupAttributes() {
    return ['id', 'name', 'description', 'email', 'query', 'code', 'ldapADsPath', 'commonName', 'distinguishedName',
      'systemIdentifier', 'isSynchronized', 'directoryId'];
  },

  validateGroupProperties(group) {
    this.getGroupAttributes().map(key => group.should.have.property(key));
  },

  validateGroupPageListProperties(groupPageList, totalCount) {
    this.validateObjectProperties(groupPageList, totalCount, this.getGroupAttributes());
  },

  getvalidateProcessParticipantProfileAttributes() {
    return ['id', 'role', 'canViewGraphicalFollowup', 'canReassignActions', 'canCancelRequests', 'canViewStatistics',
      'canReadComments', 'canWriteComments', 'query', 'processId', 'participantId'];
  },

  validateProcessParticipantProfileProperties(participant) {
    this.getvalidateProcessParticipantProfileAttributes().map(key => participant.should.have.property(key));
  },

  validateProcessParticipantProfilePageListProperties(processParticipantProfilePageList, totalCount) {
    this.validateObjectProperties(processParticipantProfilePageList, totalCount, this.getvalidateProcessParticipantProfileAttributes());
  },

  getParticipantAttributes() {
    return ['id', 'name', 'description', 'type', 'processId'];
  },

  validateParticipantProperties(participant) {
    this.getParticipantAttributes().map(key => participant.should.have.property(key));
  },

  validateParticipantPageListProperties(participantPageList, totalCount) {
    this.validateObjectProperties(participantPageList, totalCount, this.getParticipantAttributes());
  },

  getProcessAttributes() {
    return ['id', 'name', 'version', 'description', 'state', 'launchUrl', 'launchPortalUrl', 'graphicalViewUrl', 'graphicalViewPortalUrl', 'updatedAt', 'updatedById',
      'help', 'userId', 'onBehalfOf'];
  },

  validateProcessProperties(process) {
    this.getProcessAttributes().map(key => process.should.have.property(key));
  },

  validateProcessPageListProperties(processPageList, totalCount) {
    this.validateObjectProperties(processPageList, totalCount, this.getProcessAttributes());
  },

  getRequestAttributes() {
    return ['id', 'name', 'description', 'number', 'status', 'subStatus', 'isTest', 'limit', 'openedAt', 'startedAt', 'closedAt',
      'followUpFormUrl', 'followUpFormPortalUrl', 'graphicalFollowUpUrl', 'graphicalFollowUpPortalUrl', 'requesterId', 'delegatedRequesterId',
      'abortedById', 'processId', 'parentActionId', 'userId', 'onBehalfOf'];
  },

  validateRequestProperties(request) {
    this.getRequestAttributes().map(key => request.should.have.property(key));
  },

  validateRequestPageListProperties(requestPageList, totalCount) {
    this.validateObjectProperties(requestPageList, totalCount, this.getRequestAttributes());
  },

  getRequestDataAttributes() {
    return ['id', 'name', 'description', 'type', 'textValue', 'numericValue', 'dateTimeValue', 'fileValue', 'hasValue',
      'requestId', 'userId', 'onBehalfOf'];
  },

  validateRequestDataProperties(requestData) {
    this.getRequestDataAttributes().map(key => requestData.should.have.property(key));
  },

  validateRequestDataPageListProperties(requestDataPageList, totalCount) {
    this.validateObjectProperties(requestDataPageList, totalCount, this.getRequestDataAttributes());
  },

  getUserAttributes() {
    return ['id', 'userName', 'lastName', 'firstName', 'email', 'isSupervisor', 'isProcessFolderManager', 'isAdministrator', 'isArchived',
      'isActive', 'isViewer', 'defaultLanguage', 'defaultTimeZoneId', 'managerId', 'directoryId', 'isSynchronized', 'systemIdentifier'];
  },

  validateUserProperties(user) {
    this.getUserAttributes().map(key => user.should.have.property(key));
  },

  validateUserPageListProperties(userPageList, totalCount) {
    this.validateObjectProperties(userPageList, totalCount, this.getUserAttributes());
  },

  getViewAttributes() {
    return ['id', 'description', 'url', 'portalUrl', 'authorId', 'isShared', 'isPortal'];
  },

  validateViewProperties(view) {
    this.getViewAttributes().map(key => view.should.have.property(key));
  },

  validateViewPageListProperties(viewPageList, totalCount) {
    this.validateObjectProperties(viewPageList, totalCount, this.getViewAttributes());
  }
};

module.exports = utils;